/* Created on 3 janv. 2006
 *  * EMBL-EBI
 * MSD Group
 * VAMSAS Project
 * VAMAV
 * @author <a href="mailto:pierre@ebi.ac.uk">Pierre MARGUERITE</a>
 * @version  1  
 * @since 3 janv. 2006
 */
package uk.ac.ebi.msd.vamsas.vamav.documentmanagement.grouping;

import uk.ac.ebi.msd.vamsas.vamav.documentmanagement.PDBUtil;
import uk.ac.ebi.msd.vamsas.vamav.documentmanagement.grouping.exceptions.UnableToGroupSequencesException;
import uk.ac.ebi.msd.vamsas.vamav.exceptions.IncorrectVamavModuleInitialisation;
import uk.ac.ebi.msd.vamsas.vamav.session.VAMAVSession;
import uk.ac.ebi.msd.vamsas.vamav.util.data.PDBEntry;
import uk.ac.ebi.msd.vamsas.vamav.util.mapping.MappingManager;
import uk.ac.ebi.msd.vamsas.vamav.util.pool.StringBufferPool;

import java.io.File;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Properties;

/**
 * Groups sequence per PFAM domain.
 * 
 * For each sequence provided, the UniProtID is retrieved (if not provided, as source) by MSD lite.
 * Then, the UniProt ID is mapped to PFAM domains, using the sanger web server.
 * 
 * @author <a href="mailto:pierre@ebi.ac.uk">Pierre MARGUERITE</a>
 * @version 1
 * @since 3 janv. 2006 
 * 
 */
public class SequenceGrouping_domain extends SequenceGrouping
	{

		protected VAMAVSession vamavSession = null;
		
		
		/**
		 *  
		 * Constructor of the  SequenceGrouping class, given a mapping manager 
		 *
		 * @param _mappingManager the mapping manager to use for mapping 
		 */
		  public SequenceGrouping_domain(  MappingManager _mappingManager)
		    	{
		    		super(_mappingManager);
		    	}
		  
		  /**
			 *  
			 * Constructor of the  SequenceGrouping class, given a mapping manager 
			 *
			 * @param _mappingManager the mapping manager to use for mapping 
			 * @param sessionID current session for which will be used the current sequence grouping.
			 */
			  public SequenceGrouping_domain(MappingManager _mappingManager, VAMAVSession session )
			    	{
			    		super(_mappingManager);
			    		this.logger.setSession(session.getSessionID());
			    		this.vamavSession = session;
			    	}
		
		/**
		 * Inits the Sequence Grouping sub module, with a list of configuration properties.
		 */
		public void init (Properties moduleProperties) throws IncorrectVamavModuleInitialisation
			{
				super.init(moduleProperties);
			}
	    

		/** 
		 * Adds a sequence to group
		 * @see uk.ac.ebi.msd.vamsas.vamav.documentmanagement.grouping.SequenceGrouping#addSequence(java.lang.String, java.lang.String)
		 */
		@Override
		public Hashtable<String, Object> addSequence(String sequence,
				String name) throws UnableToGroupSequencesException
			{
				// TODO Auto-generated method stub
				return null;
			}
	  /**
	     * Add a sequence for grouping, with an associated accessionID and its source.
	     * 
	     * @param sequence chain sequence to add for grouping
	     * @param name name of the sequence to add
	     * @param accessionID accessionId associated to the sequence
	     * @param source source of the accessionID
	     * @return list of parameters to call the AstexViewer, if needed.  Null, otherwise.
	     * @throws UnableToGroupSequencesException if an error occurs during the adding.
	     * 
	     *  @see uk.ac.ebi.msd.vamsas.vamav.documentmanagement.grouping.SequenceGrouping#addSequence(java.lang.String, java.lang.String, java.lang.String, java.lang.String)
		 
	     */
		@Override
		public Hashtable<String, Object> addSequence(String sequence,
				String name, String accessionID, String source)
				throws UnableToGroupSequencesException
			{
				
	    		if( sequence == null || sequence.length()<1)
	    			{
	    				this.logger.sessionError("SequenceGrouping - addSequence: Can not add sequence / The sequence is unknown.");
	    				return null;
	    			}
	    		if( name == null || name.length()<1)
	    			{
	    				this.logger.sessionError("SequenceGrouping - addSequence: Can not add sequence / The name is unknown.");
	    				return null;
	    			}
	    		if( source == null || source.length()<1)
	    			{
	    				this.logger.sessionWarn("SequenceGrouping - addSequence: the accession ID source is not provided / try to determine PDBID from the sequence.");
	    				return this.addSequence(sequence, name);
	    			}
	    		if( accessionID == null || accessionID.length()<1)
	    			{
	    				this.logger.sessionWarn("SequenceGrouping - addSequence: the accession ID is not provided / try to determine PDBID from the sequence.");
	    				return this.addSequence(sequence, name);
	    			}
	    		Hashtable<String, Object> parameters =  null;
	    		try
	    			{
	    				ArrayList <PDBEntry> entries = null;
	    		
	    				if("PDB".equals(source))
	    					{
	    						PDBEntry entry = this.getMapping().getPDBEntryFromPDBID(accessionID);
	    						if( entry != null)
	    							{
	    								entries = new ArrayList<PDBEntry>(1);
	    								entries.add(entry);
	    							}
	    					}
	    				if ("UNIPROT".equalsIgnoreCase(source))
	    					{
	    						entries = this.getMapping().retrievePDBIDFromUniprotAC(accessionID);
	    					}
	    				if( entries == null || entries.size()<1) 
	    					{
	    						this.logger.sessionError("No associated PDB ID to the accessionAC "+ accessionID + " from "+source);
	    						return this.addSequence(sequence, name);
	    					}
	    				
//	    				retrieved associated domains
	    				
	    				
	    				ArrayList<String> domains = this.mapping.retrievePFAMDomain(accessionID);
	    				if (domains == null || domains.isEmpty())
	    					{
	    						logger.sessionWarn("No domain associated to "+accessionID);
	    						logger.sessionWarn("The sequence can not be grouped.");
	    						return null;
	    					}
	    				
	    				//boolean notFound = false;
	    				//hasAlignmentSequences = true;
	    				parameters = new Hashtable<String, Object>();
						
	    				int molNumber = 0;
	    				StringBuffer pdbIDs = null;
	    				if (entries != null || entries.size()<1)
	    					{
//	    						Retrieves PDB chains, before adding them to group
	    	    				//add molecule to AV parameters
	    						ArrayList<PDBUtil.Chain> associatedChains = new ArrayList<PDBUtil.Chain>();
	    						PDBUtil util = new PDBUtil(this.vamavSession);
	    						for (int entryNumber = entries.size() -1; entryNumber >=0; entryNumber --)
	    							{
	    								PDBEntry entry =  entries.get(entryNumber);
	    								
	    								parameters.put("moleculeG"+molNumber,entry);
	    								parameters.put("nameG"+molNumber, entry.getPdbID());
	    								if( !logger.isProduction()) logger.sessionDebug("Adding PDB parameters "+parameters);
	    								ArrayList<PDBUtil.Chain> chains = util.GetChain(entry);
	    								
	    								if( chains != null)
	    									{
	    										this.logger.sessionDebug(" chains found ");
	    										associatedChains.addAll(chains);
	    									}
	    								else 
	    									this.logger.sessionDebug("no chains found ");
	    					
	    								
	    								if( pdbIDs == null) pdbIDs = StringBufferPool.getString();
	    								if(pdbIDs.length()>0)pdbIDs.append(", "+entry.getPdbID());
	    								else pdbIDs.append(entry.getPdbID());
	    								//entries.remove(entry);
	    							}
	    						
	    						//treating retrieved domains
	    						for (int domainNumber = domains.size() - 1; domainNumber >=0; domainNumber --)
	    							{//add sequence to the current domain
	    								String domainID = domains.get(domainNumber);
	    								if (domainID == null || domainID.length()<0)
	    									{
	    										logger.sessionDebug("No domain. something wrong, while adding sequence to domain group.");
	    										continue;
	    									}
	    								this.addSequenceToGroup(domainID, sequence, name);
	    								//adding PDB chains to the group
	    								if( associatedChains!=null && !associatedChains.isEmpty())
	    									{
	    										for (int chainNumber = associatedChains.size()-1; chainNumber>=0; chainNumber--)
	    											{
	    												PDBUtil.Chain chain = associatedChains.get(chainNumber);
	    												if (chain == null) continue;
	    												
	    												this.addSequenceToGroup(domainID, chain.getSequence(), chain.getName());
	    											}
	    									}
	    								if( entries != null && !entries.isEmpty())
	    									{
	    										for (int entryNumber = entries.size() -1; entryNumber >=0; entryNumber --)
	    			    							{
	    			    								PDBEntry entry =  entries.get(entryNumber);
	    			    								if( entry == null) continue;
	    			    								
	    			    								this.addStructureToGroup(domainID, entry.getPdbID());
	    			    								//entries.remove(entry);
	    			    							}
	    									}
	    							}
	    						//this.addSequenceToGroup(entry.getPdbID(), sequence, name);
	    						util = null;
	    						if(!"PDB".equals(source))parameters.put(this.pdbidsproperty,pdbIDs.toString());
	    						StringBufferPool.returnString(pdbIDs);
	    						entries.clear();
	    						entries = null;
	    					}
	    				
	    			//}
	    		/*catch (IncorrectVamavModuleInitialisation inE)
	    			{
	    				logger.AppliError("Sequence Grouping addSequence - error during mapping execution. Can not proceed.", inE);
	    				throw new UnableToGroupSequencesException();
	    			}*/
	    			}
	    		catch (Exception inE)
	    			{
	    				this.logger.sessionError("Sequence Grouping addSequence - error during mapping execution. Can not proceed.", inE);
	    				throw new UnableToGroupSequencesException();
	    			}
				return parameters;
			}

		 /** Retrieves the directory to store files in.
		     * @return the output directory for created file by the sequence grouping processing.
		     */
		    public String getOutputDirectory()
		    	{
		    		File sessionDir = this.vamavSession.getSessionDirectory(); 
		    		if( sessionDir != null)
		    			return sessionDir.getAbsolutePath();
		    		return null; //should not happen, has directory file is attached during object creation. 
		    	}
		/**
		 * @see uk.ac.ebi.msd.vamsas.vamav.documentmanagement.grouping.SequenceGrouping#getSessionID()
		 */
		@Override
		public String getSessionID()
			{
				return this.vamavSession.getSessionID();
			}

	}
